@@ -1,4 +1,4 @@
-# Copyrights 2007-2013 by [Mark Overmeer].
+# Copyrights 2007-2014 by [Mark Overmeer].
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
 # Pod stripped from pm file by OODoc 2.01.
@@ -7,7 +7,7 @@ use strict;
 
 package XML::Compile::SOAP::Extension;
 use vars '$VERSION';
-$VERSION = '2.38';
+$VERSION = '3.00';
 
 use Log::Report 'xml-compile-soap';
 
@@ -33,6 +33,7 @@ sub wsdl11Init($$)
     $_->wsdl11Init(@_) for @ext;
 }
 
+#--------
 
 sub soap11OperationInit($$)
 {   ref shift and return;
@@ -55,6 +56,28 @@ sub soap11HandlerWrapper($$$)
     $cb;
 }
 
-1;
+#--------
+
+sub soap12OperationInit($$)
+{   ref shift and return;
+    $_->soap12OperationInit(@_) for @ext;
+}
+
+
+sub soap12ClientWrapper($$$)
+{   ref shift and return $_[1];
+    my ($op, $call, $args) = @_;
+    $call = $_->soap12ClientWrapper($op, $call, $args) for @ext;
+    $call;
+}
+
+
+sub soap12HandlerWrapper($$$)
+{   my ($thing, $op, $cb, $args) = @_;
+    ref $thing and return $cb;
+    $cb = $_->soap12HandlerWrapper($op, $cb, $args) for @ext;
+    $cb;
+}
+
 
 1;
